package com.loafle.overflow.central.module.infra.service;

import java.util.List;

import com.loafle.overflow.central.module.infra.dao.InfraHostIPDAO;
import com.loafle.overflow.central.module.meta.service.CentralMetaIPTypeService;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.model.infra.InfraHostIP;
import com.loafle.overflow.model.meta.MetaIPType;
import com.loafle.overflow.service.central.infra.InfraHostIPService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * Created by insanity on 17. 6. 28.
 */

@Service("InfraHostIPService")
public class CentralInfraHostIPService implements InfraHostIPService {
  @Autowired
  InfraHostIPDAO infraHostIPDAO;

  @Autowired
  CentralMetaIPTypeService metaIPTypeService;

  public InfraHostIP validate(InfraHostIP infraHostIP) throws OverflowException {
    if (null == infraHostIP) {
      throw new OverflowException("InfraHostIP is not valid");
    }

    MetaIPType metaIPType = infraHostIP.getMetaIPType();

    if (null == metaIPType) {
      throw new OverflowException("MetaIPType is not valid");
    }

    if (null == metaIPType.getId()) {
      String metaIPTypeKey = metaIPType.getKey();
      if (null == metaIPTypeKey) {
        throw new OverflowException("Key of MetaIPType is not valid");
      }

      metaIPType = this.metaIPTypeService.readByKey(metaIPTypeKey);
      if (null == metaIPType) {
        throw new OverflowException(
            String.format("MetaIPTypeKey[%s] is not valid", metaIPTypeKey));
      }
      infraHostIP.setMetaIPType(metaIPType);
    }

    if (null == infraHostIP.getAddress()) {
      throw new OverflowException("IP is not valid");
    }

    if (null == infraHostIP.getMac()) {
      throw new OverflowException("Mac is not valid");
    }

    return infraHostIP;
  }

  @Override
  public InfraHostIP regist(InfraHostIP infraHostIP) throws OverflowException {
    infraHostIP = this.validate(infraHostIP);

    if (null == infraHostIP.getInfraHost()) {
      throw new OverflowException("InfraHost is not valid");
    }

    return this.infraHostIPDAO.save(infraHostIP);
  }

  @Override
  public InfraHostIP read(Long id) throws OverflowException {
    return this.infraHostIPDAO.findById(id).get();
  }

  @Override
  public List<InfraHostIP> readByInfraHostID(Long infraHostID) throws OverflowException {
    return this.infraHostIPDAO.findAllByInfraHostId(infraHostID);
  }

  @Override
  public List<InfraHostIP> readByInfraHostIDAndMetaIPTypeKey(Long infraHostID, String metaIPTypeKey)
      throws OverflowException {
    return this.infraHostIPDAO.findAllByInfraHostIdAndMetaIPTypeKey(infraHostID, metaIPTypeKey);
  }

  @Override
  public InfraHostIP readByInfraHostIDAndMetaIPTypeKeyAndAddress(Long infraHostID, String metaIPTypeKey, String address)
      throws OverflowException {
    return this.infraHostIPDAO.findByInfraHostIdAndMetaIPTypeKeyAndAddress(infraHostID, metaIPTypeKey, address);
  }

  @Override
  public InfraHostIP readByInfraHostInfraZoneIDAndMetaIPTypeKeyAndAddress(Long infraHostInfraZoneID,
      String metaIPTypeKey, String address) throws OverflowException {
    return this.infraHostIPDAO.findByInfraHostInfraZoneIdAndMetaIPTypeKeyAndAddress(infraHostInfraZoneID, metaIPTypeKey,
        address);
  }

}
